<?php
/**
 * @version    $Id$
 * @copyright  Copyright (C) 2005 - 2011 Open Source Matters, Inc. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('JPATH_BASE') or die;

/**
 * This is our custom registration plugin class.  It verifies that the user checked the boxes
 * indicating that he/she agrees to the terms of service and is old enough to use the site.
 *
 * @package     Joomla.Plugins
 * @subpackage  User.MyRegistration
 * @since       1.0
 */
class plgUserMyRegistration extends JPlugin
{
	/**
	 * Method to handle the "onUserBeforeSave" event and determine whether we are happy with the input
	 * enough that we will allow the save to happen.  Specifically we are checking to make sure that
	 * this is saving a new user (user registration), and that the user has checked the boxes that
	 * indicate agreement to the terms of service and that he/she is old enough to use the site.
	 *
	 * @param   array  $previousData  The currently saved data for the user.
	 * @param   bool   $isNew         True if the user to be saved is new.
	 * @param   array  $futureData    The new data to save for the user.
	 *
	 * @return  bool   True to allow the save process to continue, false to stop it.
	 *
	 * @since   1.0
	 */
	public function onUserBeforeSave($previousData, $isNew, $futureData)
	{
		// If we aren't saving a "new" user (registration), then let the save happen without interruption.
		if (!$isNew || !JFactory::getApplication()->isSite()) {
			return true;
		}
		
		// Load the plugin language file
		$this->loadLanguage();

		// Verify that the "I agree to the terms of service for this site." checkbox was checked.
		if (empty($futureData['tos_agree'])) {
			JError::raiseWarning(1000, JText::_('PLG_USER_MYREGISTRATION_TOS_AGREE_REQUIRED'));
			$result = false;
		}

		// Verify that the "I am at least 18 years old." checkbox was checked.
		if (empty($futureData['old_enough'])) {
			JError::raiseWarning(1000, JText::_('PLG_USER_MYREGISTRATION_OLD_ENOUGH_REQUIRED'));
			$result = false;
		}

		return $result;
	}
	
	/**
	 * Method to handle the "onContentPrepareForm" event and alter the user registration form.  We
	 * are going to check and make sure that the form being prepared is the user registration form
	 * from the com_users component first.  If that is the form we are preparing, then we will
	 * load our custom xml file into the form object which adds our custom fields.
	 * 
	 * @param   JForm  $form  The form to be altered.
	 * @param   array  $data  The associated data for the form.
	 *
	 * @return  bool
	 * 
	 * @since   1.0
	 */
	public function onContentPrepareForm($form, $data)
	{
		// If we aren't in the registration form ignore the form.
		if ($form->getName() != 'com_users.registration') {
			return;
		}

		// Load our custom registration xml into the user registration form.
		$form->loadFile(dirname(__FILE__).'/form.xml');
	}
}
