<?php
/**
 * @package     Subscriptions
 *
 * @copyright   Copyright (C) 2011 Mark Dexter and Louis Landry. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

jimport('joomla.application.cli');
jimport('joomla.database.database');

/**
 * A Joomla Platform application to monitor subscription status and take action when necessary.
 *
 * @package  Subscriptions
 * @since    1.0
 */
class SubscriptionMonitor extends JCli
{
	/**
	 * @var    JDatabase  The application database connection object.
	 * @since  1.0
	 */
	protected $db;

	/**
	 * Class constructor.
	 *
	 * @param   mixed  $input       An optional argument to provide dependency injection for the application's
	 *                              input object.  If the argument is a JInputCli object that object will become
	 *                              the application's input object, otherwise a default input object is created.
	 * @param   mixed  $config      An optional argument to provide dependency injection for the application's
	 *                              config object.  If the argument is a JRegistry object that object will become
	 *                              the application's config object, otherwise a default config object is created.
	 * @param   mixed  $dispatcher  An optional argument to provide dependency injection for the application's
	 *                              event dispatcher.  If the argument is a JDispatcher object that object will become
	 *                              the application's event dispatcher, if it is null then the default event dispatcher
	 *                              will be created based on the application's loadDispatcher() method.
	 * @param   mixed  $db          An optional argument to provide dependency injection for the application's
	 *                              database connection.  If the argument is a JDatabase object that object will become
	 *                              the application's database connection, if it is null then the default database
	 *                              connection will be created based on the application's loadDatabase() method.
	 *
	 * @return  void
	 *
	 * @see     loadDatabase()
	 * @see     loadDispatcher()
	 * @since   1.0
	 */
	public function __construct(JInputCli $input = null, JRegistry $config = null, JDispatcher $dispatcher = null, JDatabase $db = null)
	{
		// Call the parent constructor for basic setup of the object.
		parent::__construct($input, $config, $dispatcher);

		// If a database connection object is given use it.
		if ($db instanceof JDatabase)
		{
			$this->db = $db;
		}
		// Create the database connection based on the application logic.
		else
		{
			$this->loadDatabase();
		}
			
	}

	/**
	 * Method to run the application routines.  Most likely you will want to instantiate a controller
	 * and execute it, or perform some sort of task directly.
	 *
	 * @return  void
	 *
	 * @since   1.0
	 * @throws  Exception
	 */
	protected function doExecute()
	{
		// Log start of program
		JLog::add('Subscription monitoring started');
		
 		// Get a list of the ended subscriptions and remove them.
 		$ended = $this->getEndedSubscriptions();
 		foreach ($ended as $sub)
 		{
 			$this->doEndSubscription($sub);
 		}

 		// Get a list of the ending subscriptions and send a notification.
		$ending = $this->getEndingSubscriptions();
 		foreach ($ending as $sub)
 		{
			$this->doSendSubscriptionEndingNotification($sub);
 		}
 		
 		// Log end of program
		JLog::add('Subscription monitoring ended');
	}

	/**
	 * Method to get a list of the subscriptions that have ended.
	 *
	 * @return  array
	 *
	 * @see     #__joompro_sub_mapping
	 * @since   1.0
	 */
	protected function getEndedSubscriptions()
	{
		// Get a date object for now.
		$now = JFactory::getDate($this->get('execution.datetime'));

		// Get the quey builder class from the database.
		$query = $this->db->getQuery(true);

		// Set up a query to select subscriptions that have end dates before now.
		$query->select('*')
			->from($this->db->qn('#__joompro_sub_mapping'))
			->where($this->db->qn('end_date') . ' < ' . $this->db->q($now->toMySQL()))
			->where($this->db->qn('end_date') . ' > ' . $this->db->q('0000-00-00 00:00:00'));

		// Set the database query object to the database connection object.
		$this->db->setQuery($query);

		// Get all the returned rows from the query as an array of objects.
		$rows = $this->db->loadObjectList();

		return $rows;
	}

	/**
	 * Method to get a list of the subscriptions that will be ending within the next five days.
	 *
	 * @return  array
	 *
	 * @see     #__joompro_sub_mapping
	 * @since   1.0
	 */
	protected function getEndingSubscriptions()
	{
		// Get a date object for now and five days into the future.
		$now    = JFactory::getDate($this->get('execution.datetime'));
		$future = JFactory::getDate($this->get('execution.datetime'));
		$future->add(new DateInterval('P5D'));

		// Get the quey builder class from the database.
		$query = $this->db->getQuery(true);
		$x = $this->db->quote('test');

		// Set up a query to select subscriptions that have end dates between now and 5 days from now.
		$query->select('*')
			->from($this->db->qn('#__joompro_sub_mapping'))
			->where($this->db->qn('end_date') . ' < ' . $this->db->q($future->toMySQL()))
			->where($this->db->qn('end_date') . ' > ' . $this->db->q($now->toMySQL()));

		// Set the database query object to the database connection object.
		$this->db->setQuery($query);

		// Get all the returned rows from the query as an array of objects.
		$rows = $this->db->loadObjectList();

		return $rows;
	}

 	/**
 	 * Method to end an expired subscription and send a notification email.
 	 *
 	 * @param   object  $subscriptionMapping  The subscription mapping object from the mapping database
 	 *                                        table to be ended.
 	 *
 	 * @return  void
 	 *
 	 * @see     #__joompro_sub_mapping
 	 * @since   1.0
 	 * @throws  Exception
 	 */
 	protected function doEndSubscription($subscriptionMapping)
 	{
 		$subscription = $this->getSubscription($subscriptionMapping->subscription_id);
		$user = $this->getUser($subscriptionMapping->user_id);

 		// Set up a query to remove the user group mapping.
 		$query = $this->db->getQuery(true);
 		$query->delete()
 			->from($this->db->qn('#__user_usergroup_map'))
 			->where($this->db->qn('group_id') . ' = ' . (int) $subscription->group_id)
 			->where($this->db->qn('user_id') . ' = ' . (int) $subscriptionMapping->user_id);

 		// Set the database query object to the database connection object.
 		$this->db->setQuery($query);

 		// Execute the query.
 		$this->db->query();

 		// Get the quey builder class from the database.
 		$query = $this->db->getQuery(true);

 		// Set up a query to remove the subscription mapping.
 		$query->delete()
 			->from($this->db->qn('#__joompro_sub_mapping'))
 			->where($this->db->qn('subscription_id') . ' = ' . (int) $subscriptionMapping->subscription_id)
 			->where($this->db->qn('user_id') . ' = ' . (int) $subscriptionMapping->user_id);

 		// Set the database query object to the database connection object.
 		$this->db->setQuery($query);

		// Execute the query.
 		$this->db->query();

 		// Build expiration email.
		$subject = 'Your subscription has expired.';
		$body[] = 'Hi, '.$user->name.',';
		$body[] = '';
		$body[] = 'Your subscription to '.$subscription->title.' has expired.';
		$body[] = '';
		$body[] = 'Regards,';
		$body[] = $this->get('fromname');

		// Send the notification email.
		$this->sendNotificationEmail($user->email, $subject, implode("\n", $body));
		
		// Log the removal
		JLog::add('Subscription removed for user='.$user->name.', title='.$subscription->title);
 	}

	/**
	 * Method to send a notification email for a subscription that will soon expire.
	 *
	 * @param   object  $subscriptionMapping  The subscription mapping object from the mapping database
	 *                                        table for which to send the notification.
	 *
	 * @return  void
	 *
	 * @see     #__joompro_sub_mapping
	 * @since   1.0
	 */
	protected function doSendSubscriptionEndingNotification($subscriptionMapping)
	{
		$subscription = $this->getSubscription($subscriptionMapping->subscription_id);
		$user = $this->getUser($subscriptionMapping->user_id);

 		$subject = 'Your Subscription is ending soon.';
		$body[] = 'Hi, '.$user->name.',';
		$body[] = '';
		$body[] = 'Your subscription to '.$subscription->title.' will end on '.$subscriptionMapping->end_date.'.';
		$body[] = '';
		$body[] = 'Regards,';
		$body[] = $this->get('fromname');

		// Send the notification email.
		$this->sendNotificationEmail($user->email, $subject, implode("\n", $body));
		
		// Log the notification
		JLog::add('Subscription ending notification sent for user='.$user->name.', title='.$subscription->title);
	}
	
	/**
	 * Method to get subscription data.
	 *
	 * @param   int  $subID   The subscription_id to check
	 *
	 * @return  object
	 *
	 * @codeCoverageIgnore
	 * @since   1.0
	 * @throws  Exception
	 */
	protected function getSubscription($subID) 
	{
		// Set up a query to get the group_id from the subscription record.
 		$query = $this->db->getQuery(true);
 		$query->select('*')
 			->from($this->db->qn('#__joompro_subscriptions'))
 			->where($this->db->qn('id') . ' = ' . (int) $subID);

 		// Set the database query object to the database connection object.
 		$this->db->setQuery($query);

 		// Validate that the subscription exists.
 		$subscription = $this->db->loadObject();
 		if (empty($subscription))
 		{
 			// @codeCoverageIgnoreStart
 			throw new Exception('Invalid Subscription.');
 			// @codeCoverageIgnoreEnd
 		}
 		return $subscription;	
	}
	
	/**
	 * Method to get user data
	 *
	 * @param   int  $userID   The user id to check
	 *
	 * @return  object
	 *
	 * @since   1.0
	 * @throws  Exception
	 */
	protected function getUser($userID) 
	{
	 	// Set up a query to get the user object.
 		$query = $this->db->getQuery(true);
 		$query->select('*')
 			->from($this->db->qn('#__users'))
 			->where($this->db->qn('id') . ' = ' . (int) $userID);

 		// Set the database query object to the database connection object.
 		$this->db->setQuery($query);

 		// Validate that the user exists.
 		$user = $this->db->loadObject();
 		if (empty($user))
 		{
 			// @codeCoverageIgnoreStart
 			throw new Exception('Invalid User.');
 			// @codeCoverageIgnoreEnd
 		}
 		return $user;
	}

	/**
	 * Method to isolate the sending of mail from application logic for better testability.
	 *
	 * @param   string  $email    The recipient email address.
	 * @param   string  $subject  The email subject.
	 * @param   string  $body     The email body.
	 *
	 * @return  void
	 *
	 * @codeCoverageIgnore
	 * @since   1.0
	 * @throws  Exception
	 */
	protected function sendNotificationEmail($email, $subject, $body)
	{
		JFactory::getMailer()->sendMail(
			$this->get('mailfrom'),
			$this->get('fromname'),
			$email,
			$subject,
			$body
		);
	}

	/**
	 * Method to create a database connection for the application.
	 *
	 * @return  void
	 *
	 * @since   1.0
	 * @throws  Exception
	 */
	protected function loadDatabase()
	{
		// Note, this will throw an exception if there is an error creating the database connection.
		$this->db = JDatabase::getInstance(
			array(
				'driver' => $this->get('dbtype'),
				'host' => $this->get('host'),
				'user' => $this->get('user'),
				'password' => $this->get('password'),
				'database' => $this->get('db'),
				'prefix' => $this->get('dbprefix'),
			)
		);
	}
}
